function Maps(region, map, regioncode) {
  // Array of map images
  this.maps = [];

  // Array of underlays
  this.underlays = [];
  this.legends = [];

  //Region of the world for map
  this.region = region;

  //Specific map to get
  this.map = map;

  this.locationID = null;

  //If a regional map, applicable region
  this.regioncode = regioncode;

  //URLs of maps at different zoom levels
  this.mapOptions = [];

  //Index of map being used in mapOptions
  this.currentMap = 0;

  //Current alerts for region
  this.alert = {};
  this.alert.innerHTML = null;
  this.alert.background = null;


  // Request object for URLS
  this.urlRequest = null;

  // Last time this map was updated
  this.lastUpdated = null;

  // Index in favorites
  this.index = -1;

  // Highlight box position
  this.highlight_box = [];

  this.cannotBeDisplayed = actualSource && (this.map > 0 && this.map != 16 && this.map < 69);

  // METHODS
  // Validate a region code
  this.validateRegionCode = function(code) {
    var index = 0;
    var isNumber = true;

    for (index = 0; code.length == 5 && index < code.length; index++) {
      if ((code[index] < "0" || code[index] > "9"))
        isNumber = false;
    }
    if ((code.length == 5 || code.length == 8) && isNumber)
      return true;
    else
      return false;

  };

  // Get necessary URLS
  this.getURLS = function() {
    if (actualSource == 0) weatherChannel.getURLS(this);
    else NOAA.getURLS(this);
  };


  this.getPageWithMap = function() {
    if (actualSource == 0) weatherChannel.getPageWithMap(this);
    else NOAA.getPageWithMap(this);
  };
  // If they exist, load the 300 and 100 mile maps.
  this.getOtherMaps = function() {
    if (actualSource == 0) weatherChannel.getOtherMaps(this);
    else NOAA.getOtherMaps(this);
  };

  this.getMaps = function() {
    if (actualSource == 0) weatherChannel.getMaps(this);
    else NOAA.getMaps(this);
  };

  this.removeOldMaps = function() {
    var counter = 0;
    while (counter < this.maps.length && this.isOlder(counter)) {
      counter++;
    }

    // Make sure there are at least five maps
    if (counter > 0) {
      counter = (this.maps.length - counter < 5) ? (this.maps.length - 5) : counter;
      for (mapIndex = 0; mapIndex < counter; mapIndex++) {
        trash(this.maps[mapIndex]);
        if (this.legends.length > 0) trash(this.legends[mapIndex]);
      }
      this.maps.splice(0, counter);
      if (this.legends.length > 0) this.legends.splice(0, counter);
    }
  };

  this.storeMapURL = function(str, mapdest) {
    var mapOptionsIndex = this.mapOptions.length;
    this.mapOptions[mapOptionsIndex] = [];
    this.mapOptions[mapOptionsIndex][0] = str;
    this.mapOptions[mapOptionsIndex][1] = mapdest;

  };

  this.storeForecastMapURL = function(mapsPage) {
    var newImage = [];
    var imgIndex = 0;
    var allImages = mapsPage.body.getElementsByTagName("img");
    newImage = (allImages[imgIndex++]).src;
    while (newImage.indexOf("forecast") == -1) newImage = (allImages[imgIndex++]).src;
    var mapOptionsIndex = this.mapOptions.length;
    this.mapOptions[mapOptionsIndex] = [];
    this.mapOptions[mapOptionsIndex][0] = "Forecast";
    this.mapOptions[mapOptionsIndex++][1] = newImage;
    this.mapOptions[mapOptionsIndex] = [];
    this.mapOptions[mapOptionsIndex][0] = "Temps";
    this.mapOptions[mapOptionsIndex++][1] = newImage.replace("wkpln", "hitmpf");
    this.mapOptions[mapOptionsIndex] = [];
    this.mapOptions[mapOptionsIndex][0] = "Wind";
    this.mapOptions[mapOptionsIndex][1] = newImage.replace("wkpln", "wnd");

  };

  this.addToFavorites = function(favindex) {
    if (window.widget) {
      widget.setPreferenceForKey(this.map, createkey("map" + favindex + "map"));
      widget.setPreferenceForKey(this.region, createkey("map" + favindex + "region"));
      widget.setPreferenceForKey(this.regioncode, createkey("map" + favindex + "regioncode"));
      widget.setPreferenceForKey(this.currentMap, createkey("map" + favindex + "current"));

      if (this.highlight_box != null) {
        widget.setPreferenceForKey(this.highlight_box[0], createkey("map" + favindex + "highlightleft"));
        widget.setPreferenceForKey(this.highlight_box[1], createkey("map" + favindex + "highlighttop"));
      }

    }
    this.index = favindex;
  };

  this.removeFromFavorites = function() {
    if (window.widget) {
      widget.setPreferenceForKey(null, createkey("map" + this.index + "map"));
      widget.setPreferenceForKey(null, createkey("map" + this.index + "region"));
      widget.setPreferenceForKey(null, createkey("map" + this.index + "regioncode"));
      widget.setPreferenceForKey(null, createkey("map" + this.index + "current"));
    }
    this.index = -1;
  };

  this.gotoNextDetailMap = function() {
    this.currentMap = (this.currentMap + 1) % this.mapOptions.length;
    widget.setPreferenceForKey(this.currentMap, createkey("map" + this.index + "current"));
    this.lastUpdated = null;
    this.maps = [];
    this.legends = [];
    //document.getElementById("warning").style.visibility = "visible";
    //document.getElementById("warning").style.background = "#283f8d";
    //document.getElementById("warning").innerHTML = "Loading Maps";
    this.getMaps();
    if (this.underlays.length > 0) {
      document.getElementById("topoimg").src = this.underlays[this.currentMap][0];
      document.getElementById("countiesimg").src = this.underlays[this.currentMap][1];
      document.getElementById("highwaysimg").src = this.underlays[this.currentMap][2];
      document.getElementById("citiesimg").src = this.underlays[this.currentMap][3];
    } else {
      document.getElementById("topoimg").src = "blank.png";
      document.getElementById("countiesimg").src = "blank.png";
      document.getElementById("highwaysimg").src = "blank.png";
      document.getElementById("citiesimg").src = "blank.png";
      document.getElementById("legendimg").src = "blank.png";
    }
    //document.getElementById("warning").style.visibility = "hidden";

  };

  this.getAlerts = function() {
    NOAA.getAlerts(this);
  };
  this.isOlder = function(counter) {
    if (actualSource == 0)
      return this.lastUpdated.getTime() - this.maps[counter].substring(this.maps[counter].indexOf("-maps") + 6, this.maps[counter].length - 4) > actualSaveMaps;
    else
      return this.lastUpdated == null || this.lastUpdated.getTime() - NOAA.filenameToTime(this.maps[counter]).getTime() > actualSaveMaps;
  };

  this.getURLS();

  this.saveHighlight = function() {
    if (this.highlight_box != null) {
      widget.setPreferenceForKey(this.highlight_box[0], createkey("map" + this.index + "highlightleft"));
      widget.setPreferenceForKey(this.highlight_box[1], createkey("map" + this.index + "highlighttop"));
    } else {
      widget.setPreferenceForKey(null, createkey("map" + this.index + "highlightleft"));
      widget.setPreferenceForKey(null, createkey("map" + this.index + "highlighttop"));
    }
  };
}