// Program version
var versionNum = "2.8.1";

// Default values
var defaultMap = 1;
var defaultZipCode = 14850;
var defaultAnimationSpeed = 250;
var defaultUpdateFrequency = 10;
var defaultRegion = 0;
var defaultMapToGet = 0;
var defaultMaximized = false;
var defaultHold = false;
var defaultStopped = false;
var defaultBorder = true;
var defaultActivateUpdate = -1;
var defaultRotateTime = 10;
var defaultShowAlerts = true;
var defaultSwitchAtEnd = false;
var defaultSaveMaps = 0;
var defaultFavorites = 0;
var defaultSource = 0;
var defaultForecastHalf = false;

// Real values:
var actualAnimationSpeed = defaultAnimationSpeed;
var actualUpdateFrequency = defaultUpdateFrequency;
var actualHold = defaultHold;
var actualStopped = defaultStopped;
var actualBorder = defaultBorder;
var actualActivateUpdate = defaultActivateUpdate;
var actualShowAlerts = defaultShowAlerts;
var actualSaveMaps = defaultSaveMaps;
var actualSwitchAtEnd = defaultSwitchAtEnd;
var actualFavorites = defaultFavorites;
var actualRotateTime = defaultRotateTime;
var actualSource = defaultSource;
var actualForecastHalf = defaultForecastHalf;

//Storage of maps
var currentMap = 0;
var imgCounter = 0;
var allMaps = [];

var windowWidth;
var windowHeight;
var userName = "unknown";
if (window.widget && widget.system("/bin/ls /usr/bin/id", null).outputString)
  userName = widget.system("echo $HOME", null).outputString.slice(0, -1);
var folder = userName + "/Library/Application Support/RadarInMotion";

// Null when not running, i.e., animation should be stopped
var timer = null;
var switchTimer = null;
var updateTimer = null;

var lastUpdated;

var active = true;


// Elements in document for quicker reference
var warningPane = null;
var mapSwitchButton = null;
var statusPane = null;

function setup() {
  // Get document elements
  warningPane = document.getElementById("warning");
  mapSwitchButton = document.getElementById("map600Button");
  statusPane = document.getElementById("status");

  if (window.widget && widget.system("/bin/ls /usr/bin/defaults", null).outputString) {
    var addressBookRegion = widget.system("/usr/bin/defaults read AddressBookMe ZipPostal", null).outputString;
    if (addressBookRegion && addressBookRegion != null && validateZIPCode(addressBookRegion.substring(0, 5))) {
      defaultZipCode = addressBookRegion.substring(0, 5);
      defaultMap = 0;
    }
  }
  statusPane.style.visibility = "visible";
  statusPane.style.width = "0px";

  warningPane.innerHTML = "Loading Widget.";
  // Try to retrieve preferences, if they exist
  if (window.widget) {
    actualAnimationSpeed = widget.preferenceForKey(createkey("animationspeed"));
    actualUpdateFrequency = widget.preferenceForKey(createkey("updatefrequency"));
    actualHold = widget.preferenceForKey(createkey("hold"));
    actualForecastHalf = widget.preferenceForKey(createkey("forecastspeed"));
    actualStopped = widget.preferenceForKey(createkey("stopped"));
    actualBorder = widget.preferenceForKey(createkey("hasborder"));
    actualActivateUpdate = widget.preferenceForKey(createkey("activateupdate"));
    actualRotateTime = widget.preferenceForKey(createkey("rotatetime"));
    actualShowAlerts = widget.preferenceForKey(createkey("showalerts"));
    actualSwitchAtEnd = widget.preferenceForKey(createkey("switchatend"));
    actualSaveMaps = widget.preferenceForKey(createkey("savemaps"));
    actualSource = widget.preferenceForKey(createkey("source"));
    actualFavorites = widget.preferenceForKey(createkey("favorites"));
    windowHeight = widget.preferenceForKey(createkey("windowheight"));
    windowWidth = widget.preferenceForKey(createkey("windowwidth"));
    warningPane.innerHTML = "Loading Widget.";
    // If any values were not able to be retrieved, set them to defaults
    // and set preferences
    if (actualAnimationSpeed == "" || actualAnimationSpeed == null || !validateNumber(actualAnimationSpeed)) {
      actualAnimationSpeed = defaultAnimationSpeed;
      widget.setPreferenceForKey(actualAnimationSpeed, createkey("animationspeed"));
    }

    if (actualUpdateFrequency == null) {
      actualUpdateFrequency = defaultUpdateFrequency;
      widget.setPreferenceForKey(actualUpdateFrequency, createkey("updatefrequency"));
    }

    if (actualFavorites == null || isNaN(actualFavorites)) {
      actualFavorites = defaultFavorites;
      widget.setPreferenceForKey(actualFavorites, createkey("favorites"));
    }

    if (actualHold == null) {
      actualHold = defaultHold;
      widget.setPreferenceForKey(actualHold, createkey("hold"));
    }

    if (actualForecastHalf == null) {
      actualForecastHalf = defaultForecastHalf;
      widget.setPreferenceForKey(actualHold, createkey("forecastspeed"));
    }

    if (actualStopped == null) {
      actualStopped = defaultStopped;
      widget.setPreferenceForKey(actualStopped, createkey("stopped"));
    }

    if (actualBorder == null) {
      actualBorder = defaultBorder;
      widget.setPreferenceForKey(actualBorder, createkey("hasborder"));
    }

    if (actualShowAlerts == null) {
      actualShowAlerts = defaultShowAlerts;
      widget.setPreferenceForKey(actualShowAlerts, createkey("showalerts"));
    }

    if (actualSwitchAtEnd == null) {
      actualSwitchAtEnd = defaultSwitchAtEnd;
      widget.setPreferenceForKey(actualSwitchAtEnd, createkey("switchatend"));
    }

    if (actualActivateUpdate == "" || actualActivateUpdate == null) {
      actualActivateUpdate = defaultActivateUpdate;
      widget.setPreferenceForKey(actualActivateUpdate, createkey("activateupdate"));
    }

    if (actualRotateTime == null) {
      actualRotateTime = defaultRotateTime;
      widget.setPreferenceForKey(actualRotateTime, createkey("rotatetime"));
    }

    if (actualSaveMaps == null) {
      actualSaveMaps = defaultSaveMaps;
      widget.setPreferenceForKey(actualSaveMaps, createkey("savemaps"));
    }

    if (actualSource == null) {
      actualSource = defaultSource;
      widget.setPreferenceForKey(actualSource, createkey("source"));
    }

    if (actualSaveMaps > 0) {
      createDirectory();
      if (widget.system("/bin/ls \"" + folder + "\"", null).errorString) actualSaveMaps = 0;
    }

    var nextFavMap = null;
    if (actualFavorites > 0) {
      var foundFaves = 0;
      // Find all of the favorites
      for (favind = 0; foundFaves < actualFavorites; favind++) {
        nextFavMap = widget.preferenceForKey(createkey("map" + favind + "region"));
        if (nextFavMap != undefined) {
          var nextFavMapLeftHighlight = widget.preferenceForKey(createkey("map" + favind + "highlightleft"));
          var nextFavMapTopHighlight = widget.preferenceForKey(createkey("map" + favind + "highlighttop"));
          allMaps[foundFaves] = new Maps(nextFavMap, widget.preferenceForKey(createkey("map" + favind + "map")),
            widget.preferenceForKey(createkey("map" + favind + "regioncode")));
          allMaps[foundFaves].index = favind;
          allMaps[foundFaves].currentMap = widget.preferenceForKey(createkey("map" + favind + "current"));
          if (nextFavMapLeftHighlight != undefined)
            allMaps[foundFaves].highlight_box = [nextFavMapLeftHighlight, nextFavMapTopHighlight];
          //Clean up numbers
          allMaps[foundFaves].removeFromFavorites();
          allMaps[foundFaves].addToFavorites(foundFaves++);
        }
      }
    } else {
      nextFavMap = widget.preferenceForKey(createkey("map-1region"));
      if (nextFavMap != undefined) {
        allMaps[0] = new Maps(nextFavMap, widget.preferenceForKey(createkey("map-1map")),
          widget.preferenceForKey(createkey("map-1regioncode")));
        allMaps[0].index = -1;
        allMaps[0].currentMap = widget.preferenceForKey(createkey("map-1current"));
      } else allMaps[0] = new Maps(defaultRegion, defaultMap, defaultZipCode);
    }
  }
  warningPane.innerHTML = "Loading Widget..";
  // Make sure widget size and border are correct
  if (windowHeight == "" || windowHeight == null) {
    windowWidth = 390;
    windowHeight = 295;
  }

  updateBorder();
  // Load initial map
  document.getElementById("maps").innerHTML = "<img id=\"currentmap\" src=\"\"  ondblclick=\"maximize(event);\" onclick=\"movebox();\" />";
  resizeWidgetTo(windowWidth, windowHeight);
  warningPane.innerHTML = "Loading Widget...";
  update();
  updateMapNumber();
  updateHighlightBox();
  warningPane.style.visibility = "visible";
  if (warningPane.innerHTML.indexOf("Warning:") == -1) {
    warningPane.innerHTML = "Loading Widget....";
  }
  if (allMaps[currentMap].mapOptions.length > 1) {
    mapSwitchButton.style.visibility = "visible";
    mapSwitchButton.children[1].innerHTML = allMaps[currentMap].mapOptions[(allMaps[currentMap].currentMap + 1) % allMaps[currentMap].mapOptions.length][0];
  } else {
    mapSwitchButton.style.visibility = "hidden";
  }

  if (allMaps[currentMap].index > -1) {
    document.getElementById("removefromfavesButton").style.visibility = "visible";
    document.getElementById("addtofavesButton").style.visibility = "hidden";
  } else {
    document.getElementById("removefromfavesButton").style.visibility = "hidden";
    document.getElementById("addtofavesButton").style.visibility = "visible";
  }

  if (allMaps[currentMap].mapOptions.length != 0) {
    document.getElementById("maps").innerHTML = "<img id=\"currentmap\" src=\"" +
      allMaps[currentMap].maps[0] + "\"  ondblclick=\"maximize(event);\"/>";
  }
  resizeWidgetTo(windowWidth, windowHeight);
  if (warningPane.innerHTML.indexOf("Warning:") == -1) warningPane.innerHTML = "Loading Widget.....";
  // Make sure manual buttons are correct
  if (actualStopped) {
    document.getElementById("startButton").style.visibility = "visible";
    document.getElementById("stopButton").style.visibility = "hidden";
  }

  // If using NOAA, do not allow selection of other countries
  if (actualSource != 0) {
    document.getElementById("region").innerHTML = "<option>United States</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Northeast</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--East Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Southeast</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--North Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--South Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Northwest</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--West Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Southwest</option>";
    document.getElementById("weatherURL").style.visibility = "hidden";
  }

  // Start animation
  stop();
  if (active) start();


  // If necessary, create timer for updating widget
  if (updateTimer == null && actualUpdateFrequency != 0)
    updateTimer = setInterval("update();", actualUpdateFrequency * 60 * 1000);

  //warningPane.style.visibility = "hidden";
  switchToMap(0, true);
}



// Select region of the country based on region value and set the body of the maptype select group
function selectedRegion() {
  switch (document.getElementById("region").selectedIndex) {
    // United States
    case 0:
      document.getElementById("maptype").innerHTML = (actualSource == 0) ? "<option value='0'>Regional Radar (enter city)</option>" +
        "<option value='1'>US Radar</option>" +
        "<option value='16'>US Satellite</option>" +
        "<option value='29'>US Visible Satellite</option>" +
        "<option value='39'>US Water Vapor Satellite</option>" +
        "<option value='69'>US Forecast</option>" :
        "<option value='0'>Regional Radar (enter city)</option>" +
        "<option value='16'>US Satellite</option>" +
        "<option value='69'>US Forecast</option>";

      break;
    case 1:
      document.getElementById("maptype").innerHTML =
        ((actualSource == 0) ? ("<option value='0'>Regional Radar (enter city)</option>" +
          "<option value='2'>US Northeast 2100mi Radar</option>" +
          "<option value='7'>US Northeast 1800mi Radar</option>" +
          "<option value='17'>US Northeast Satellite</option>" +
          "<option value='30'>US Northeast Visible Satellite</option>") : ("")) +
        "<option value='70'>US Northeast Forecast</option>" +
        "<option value='79'>US Northeast Temperature Forecast</option>" +
        "<option value='88'>US Northeast Wind Forecast</option>";
      break;
    case 2:
      document.getElementById("maptype").innerHTML =
        ((actualSource == 0) ? ("<option value='0'>Regional Radar (enter city)</option>" +
          "<option value='8'>US East Central 1800mi Radar</option>" +
          "<option value='18'>US East Central Satellite</option>" +
          "<option value='31'>US East Central Visible Satellite</option>") : ("")) +
        "<option value='71'>US East Central Forecast</option>" +
        "<option value='80'>US East Central Temperature Forecast</option>" +
        "<option value='89'>US East Central Wind Forecast</option>";
      break;
    case 3:
      document.getElementById("maptype").innerHTML =
        ((actualSource == 0) ? ("<option value='0'>Regional Radar (enter city)</option>" +
          "<option value='3'>US Southeast 2100mi Radar</option>" +
          "<option value='9'>US Southeast 1800mi Radar</option>" +
          "<option value='19'>US Southeast Satellite</option>" +
          "<option value='32'>US Southeast Visible Satellite</option>") : ("")) +
        "<option value='72'>US Southeast Forecast</option>" +
        "<option value='81'>US Southeast Temperature Forecast</option>" +
        "<option value='90'>US Southeast Wind Forecast</option>";
      break;
    case 4:
      document.getElementById("maptype").innerHTML =
        ((actualSource == 0) ? ("<option value='0'>Regional Radar (enter city)</option>" +
          "<option value='4'>US Midwest 2100mi Radar</option>" +
          "<option value='10'>US North Central 1800mi Radar</option>" +
          "<option value='20'>US North Central Satellite</option>" +
          "<option value='33'>US North Central Visible Satellite</option>") : ("")) +
        "<option value='73'>US North Central Forecast</option>" +
        "<option value='82'>US North Central Temperature Forecast</option>" +
        "<option value='91'>US North Central Wind Forecast</option>";
      break;
    case 5:
      document.getElementById("maptype").innerHTML =
        ((actualSource == 0) ? ("<option value='0'>Regional Radar (enter city)</option>" +
          "<option value='4'>US Midwest 2100mi Radar</option>" +
          "<option value='11'>US Central 1800mi Radar</option>" +
          "<option value='21'>US Central Satellite</option>" +
          "<option value='34'>US Central Visible Satellite</option>") : ("")) +
        "<option value='74'>US Central Forecast</option>" +
        "<option value='83'>US Central Temperature Forecast</option>" +
        "<option value='92'>US Central Wind Forecast</option>";
      break;
    case 6:
      document.getElementById("maptype").innerHTML =
        ((actualSource == 0) ? ("<option value='0'>Regional Radar (enter city)</option>" +
          "<option value='4'>US Midwest 2100mi Radar</option>" +
          "<option value='12'>US South Central 1800mi Radar</option>" +
          "<option value='22'>US South Central Satellite</option>" +
          "<option value='35'>US South Central Visible Satellite</option>") : ("")) +
        "<option value='75'>US South Central Forecast</option>" +
        "<option value='84'>US South Central Temperature Forecast</option>" +
        "<option value='93'>US South Central Wind Forecast</option>";
      break;
    case 7:
      document.getElementById("maptype").innerHTML =
        ((actualSource == 0) ? ("<option value='0'>Regional Radar (enter city)</option>" +
          "<option value='26'>US West Coast Satellite</option>" +
          "<option value='5'>US West 2100mi Radar</option>" +
          "<option value='13'>US Northwest 1800mi Radar</option>" +
          "<option value='23'>US Northwest Satellite</option>" +
          "<option value='36'>US Northwest Visible Satellite</option>") : ("")) +
        "<option value='76'>US Northwest Forecast</option>" +
        "<option value='85'>US Northwest Temperature Forecast</option>" +
        "<option value='94'>US Northwest Wind Forecast</option>";
      break;
    case 8:
      document.getElementById("maptype").innerHTML =
        ((actualSource == 0) ? ("<option value='0'>Regional Radar (enter city)</option>" +
          "<option value='26'>US West Coast Satellite</option>" +
          "<option value='14'>US West Central 1800mi Radar</option>" +
          "<option value='24'>US West Central Satellite</option>" +
          "<option value='37'>US West Central Visible Satellite</option>") : ("")) +
        "<option value='77'>US West Central Forecast</option>" +
        "<option value='86'>US West Central Temperature Forecast</option>" +
        "<option value='95'>US West Central Wind Forecast</option>";
      break;
    case 9:
      document.getElementById("maptype").innerHTML =
        ((actualSource == 0) ? ("<option value='0'>Regional Radar (enter city)</option>" +
          "<option value='26'>US West Coast Satellite</option>" +
          "<option value='38'>US Southwest Visible Satellite</option>" +
          "<option value='15'>US Southwest 1800mi Radar</option>" +
          "<option value='25'>US Southwest Satellite</option>") : ("")) +
        "<option value='78'>US Southwest Forecast</option>" +
        "<option value='87'>US Southwest Temperature Forecast</option>" +
        "<option value='96'>US Southwest Wind Forecast</option>";
      break;
    case 10:
      document.getElementById("maptype").innerHTML =
        "<option value='0'>Regional Radar (enter city)</option>" +
        "<option value='6'>US Alaska 2100mi Radar</option>" +
        "<option value='27'>US Alaska Satellite</option>" +
        "<option value='28'>US Hawaii Satellite</option>";
      break;
      // Africa & Middle East
    case 11:
      document.getElementById("maptype").innerHTML = "<option value='55'>Africa Satellite</option>" +
        "<option value='56'>Middle East Satellite</option>";
      break;

      // Asia
    case 12:
      document.getElementById("maptype").innerHTML = "<option value='49'>Asia Satellite</option>" +
        "<option value='50'>Central Asia Satellite</option>" +
        "<option value='51'>East Asia Satellite</option>" +
        "<option value='52'>Afghanistan Satellite</option>" +
        "<option value='53'>India Satellite</option>" +
        "<option value='54'>Japan Satellite</option>";

      break;

      // Australia
    case 13:
      document.getElementById("maptype").innerHTML = "<option value='47'>Australia Satellite</option>" +
        "<option value='48'>New Zealand Satellite</option>";
      break;

      // Central America
    case 14:
      document.getElementById("maptype").innerHTML = "<option value='57'>Central America Satellite</option>" +
        "<option value='68'>Caribbean Satellite</option>";
      break;

      // Europe
    case 15:
      document.getElementById("maptype").innerHTML = "<option value='40'>European Satellite</option>" +
        "<option value='41'>UK Satellite</option>" +
        "<option value='42'>France Satellite</option>" +
        "<option value='43'>Italy Satellite</option>" +
        "<option value='44'>Germany Satellite</option>" +
        "<option value='45'>Russia Satellite</option>" +
        "<option value='46'>Scandinavia Satellite</option>";
      break;

      // Global
    case 16:
      document.getElementById("maptype").innerHTML = "<option value='63'>Pacific Global Satellite</option>" +
        "<option value='64'>Europe/Africa Global Satellite</option>" +
        "<option value='65'>Tropical Atlantic Satellite</option>" +
        "<option value='66'>Atlantic Ocean Satellite</option>";
      break;

      // North America
    case 17:
      document.getElementById("maptype").innerHTML =
        "<option value='67'>Gulf of Mexico Satellite</option>" +
        "<option value='58'>Canada Satellite</option>" +
        "<option value='59'>Mexico Satellite</option>";
      break;

      // Pacific Ocean
    case 18:
      document.getElementById("maptype").innerHTML = "<option value='60'>Pacific Ocean Satellite</option>";
      break;

      // South America
    case 19:
      document.getElementById("maptype").innerHTML = "<option value='61'>South America Satellite</option>" +
        "<option value='62'>Brazil Satellite</option>";
      break;

  }

  // Change state of ZIP code box if necessary
  selectedIsZip();

}

// Make city box active if "Regional Radar" is element selected in maps
function selectedIsZip() {
  if (document.getElementById("maptype").value == 0)
    document.getElementById("zipcode").disabled = false;
  else
    document.getElementById("zipcode").disabled = true;
}

/**********
 * FILE MANAGEMENT FUNCTIONS
 **********/
// Create Necessary Directories
function createDirectory() {
  if (window.widget && !widget.system("/bin/ls \"" + folder + "\"", null).outputString)
    widget.system("/bin/mkdir \"" + folder + "\"", null);
  if (folder.indexOf("-maps") == -1) folder = folder + "/" + createkey("maps");
  if (window.widget && widget.system("/bin/ls \"" + folder + "\"", null).outputString) {
    trash(folder);
  }
  if (window.widget) widget.system("/bin/mkdir \"" + folder + "\"", null);
}

function clearMaps() {
  for (j = 0; actualSaveMaps > 0 && j < allMaps.length; j++) {
    for (i = 0; actualSaveMaps > 0 && i < allMaps[j].maps.length; i++) {
      if (allMaps[j].maps[i].indexOf("http:") == -1) {
        trash(allMaps[j].maps[i]);

      }
    }
    allMaps[j].maps = [];
    allMaps[j].lastUpdated = null;
  }
  imgCounter = 0;
  lastUpdated = null;
}

// Move saved maps to trash
function trash(file) {
  //alert("Trashing" + file);
  var trashFolder = userName + "/.Trash/RadarInMotion";
  //if(!widget.system("/bin/ls \"" + trashFolder + "\"",null).outputString)
  //  widget.system("/bin/mkdir \"" + trashFolder + "\"/",null);
  //widget.system("/bin/mv -f \"" + file + "\" \"" + trashFolder + "/" + (new Date()).getTime() + "\"",null);
  widget.system("/bin/rm -rf \"" + file + "\"", null);
}

// Default functions for behavior when Dashboard is activated/deactivated
if (window.widget) {
  widget.onhide = onhide;
  widget.onshow = onshow;
  widget.onremove = onremove;
}

function onshow() {
  if (actualActivateUpdate != -1) {
    now = new Date();
    if ((now.getTime() - lastUpdated.getTime()) / (1000 * 60) > actualActivateUpdate)
      update();
  }
  if (warningPane.innerHTML && warningPane.innerHTML.indexOf("Loading") != -1)
    warningPane.style.visibility = "hidden";
  start();
  active = true;
}

function onhide() {
  stop();
  active = false;
}

function onremove() {
  if (folder.indexOf("-maps") != -1) trash(folder);
  widget.setPreferenceForKey(null, createkey("animationspeed"));
  widget.setPreferenceForKey(null, createkey("updatefrequency"));
  widget.setPreferenceForKey(null, createkey("favorites"));
  widget.setPreferenceForKey(null, createkey("hold"));
  widget.setPreferenceForKey(null, createkey("forecastspeed"));
  widget.setPreferenceForKey(null, createkey("stopped"));
  widget.setPreferenceForKey(null, createkey("hasborder"));
  widget.setPreferenceForKey(null, createkey("showalerts"));
  widget.setPreferenceForKey(null, createkey("switchatend"));
  widget.setPreferenceForKey(null, createkey("activateupdate"));
  widget.setPreferenceForKey(null, createkey("rotatetime"));
  widget.setPreferenceForKey(null, createkey("savemaps"));
  widget.setPreferenceForKey(null, createkey("source"));
  widget.setPreferenceForKey(null, createkey("windowwidth"));
  widget.setPreferenceForKey(null, createkey("windowheight"));
  for (j = -1; j < actualFavorites; j++) {
    widget.setPreferenceForKey(null, createkey("map" + j + "map"));
    widget.setPreferenceForKey(null, createkey("map" + j + "regioncode"));
    widget.setPreferenceForKey(null, createkey("map" + j + "region"));
    widget.setPreferenceForKey(null, createkey("map" + j + "current"));
  }


}

// Stop animation
function stop() {
  if (timer != null) {
    clearInterval(timer);
    timer = null;
  }
  if (switchTimer != null) {
    clearInterval(switchTimer);
    switchTimer = null;
  }
}

// Start animation
function start() {
  if (timer == null && !actualStopped) {
    timer = (actualForecastHalf && allMaps[currentMap].map >= 69) ? setInterval("animateMap();", actualAnimationSpeed * 2) :
      setInterval("animateMap();", actualAnimationSpeed);
    if (actualRotateTime > 0 && switchTimer == null)
      switchTimer = setInterval("switchMap();", 1000 * actualRotateTime);
  }

}

// Flipping animation: From Apple Developer Website
var flipShown = false;
var animation = {
  duration: 0,
  starttime: 0,
  to: 1.0,
  now: 0.0,
  from: 0.0,
  firstElement: null,
  timer: null
};

function mousemove(event) {
  if (!flipShown) {
    if (animation.timer != null) {
      clearInterval(animation.timer);
      animation.timer = null;
    }

    var starttime = (new Date()).getTime() - 13;

    animation.duration = 500;
    animation.starttime = starttime;
    animation.firstElement = document.getElementById('flip');
    animation.timer = setInterval("animate();", 13);
    animation.from = animation.now;
    animation.to = 1.0;
    animate();
    flipShown = true;
  }
}

function mouseexit(event) {
  if (flipShown) {
    // fade in the info button
    if (animation.timer != null) {
      clearInterval(animation.timer);
      animation.timer = null;
    }

    var starttime = (new Date()).getTime() - 13;

    animation.duration = 500;
    animation.starttime = starttime;
    animation.firstElement = document.getElementById('flip');
    animation.timer = setInterval("animate();", 13);
    animation.from = animation.now;
    animation.to = 0.0;
    animate();
    flipShown = false;

    hideRefresh();
  }

  document.getElementById('fliprollieblack').style.display = 'none';
}

function animate() {
  var T;
  var ease;
  var time = (new Date()).getTime();


  T = limit_3(time - animation.starttime, 0, animation.duration);

  if (T >= animation.duration) {
    clearInterval(animation.timer);
    animation.timer = null;
    animation.now = animation.to;
  } else {
    ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
    animation.now = computeNextFloat(animation.from, animation.to, ease);
  }

  animation.firstElement.style.opacity = animation.now;
}

function limit_3(a, b, c) {
  return a < b ? b : (a > c ? c : a);
}

function computeNextFloat(from, to, ease) {
  return from + (to - from) * ease;
}

// Get the next image and display it
// Hold most recent map for five clicks if last map is to be held
var holdSteps = 5;
imgCounter = 0;
var isASwitch = true;

function animateMap() {
  var currentMapsArray = allMaps[currentMap].maps;
  var currentMapsArraylength = currentMapsArray.length;
  if (currentMapsArray.length > 0) {
    //if(isNaN(imgCounter)) imgCounter = 0;
    if (imgCounter != (currentMapsArraylength - 1) || !actualHold || holdSteps-- == 0 || allMaps[currentMap].map >= 69) {
      if (holdSteps < 0) holdSteps = 5;
      if (imgCounter == currentMapsArraylength - 1 && actualSwitchAtEnd && isASwitch) {
        isASwitch = false;
        switchMap();
        isASwitch = true;
      } else {
        imgCounter = (imgCounter + 1) % currentMapsArraylength;
        if (document.getElementById("currentmap") != null) {
          document.getElementById("currentmap").src = currentMapsArray[imgCounter];
          if (allMaps[currentMap].legends.length != 0) document.getElementById("legendimg").src = allMaps[currentMap].legends[imgCounter];
        }
        statusPane.style.width = (statusWidth / currentMapsArraylength) * (imgCounter + 1) + "px";
      }
    }

  }

}

function animateMapBackwards() {
  var currentMapsArray = allMaps[currentMap].maps;
  var currentMapsArraylength = currentMapsArray.length;
  if (currentMapsArray.length > 0) {
    if (imgCounter == 0 && actualSwitchAtEnd && isASwitch) {
      isASwitch = false;
      switchMapUp();
      isASwitch = true;
    } else {
      //if(isNaN(imgCounter)) imgCounter = 0;
      //if(imgCounter != (allMaps[currentMap].maps.length - 1) || !actualHold || holdSteps-- == 0)
      imgCounter = (imgCounter + (currentMapsArraylength - 1)) % currentMapsArraylength;
      if (document.getElementById('currentmap') != null) {
        document.getElementById('currentmap').src = currentMapsArray[imgCounter];
        if (allMaps[currentMap].legends.length != 0) document.getElementById("legendimg").src = allMaps[currentMap].legends[imgCounter];
      }
      statusPane.style.width = (statusWidth / currentMapsArraylength) * (imgCounter + 1) + "px";
    }
  }

}

function reloadCurrentMap() {
  var currentMapsArraylength = allMaps[currentMap].maps.length;
  if (imgCounter > currentMapsArraylength) imgCounter = 0;
  if (document.getElementById("currentmap") != null)
    document.getElementById("currentmap").src = allMaps[currentMap].maps[imgCounter];
  statusPane.style.width = (statusWidth / currentMapsArraylength) * (imgCounter + 1) + "px";
}

// Stop button on front of widget was pressed
function manualStop() {
  document.getElementById("startButton").style.visibility = "visible";
  document.getElementById("stopButton").style.visibility = "hidden";
  stop();
  actualStopped = true;
  if (window.widget)
    widget.setPreferenceForKey(actualStopped, createkey("stopped"));

}

// Start button on front of widget was pressed
function manualStart() {
  actualStopped = false;
  if (window.widget)
    widget.setPreferenceForKey(actualStopped, createkey("stopped"));
  if (active) start();
  document.getElementById("startButton").style.visibility = "hidden";
  document.getElementById("stopButton").style.visibility = "visible";
}

// Next button on front of widget was pressed
function nextMap() {
  var hold = actualHold;
  actualHold = false;
  animateMap();
  actualHold = hold;
}

// Last button on front of widget was pressed
function lastMap() {
  var hold = actualHold;
  actualHold = false;
  animateMapBackwards();
  actualHold = hold;
}

function switchToMap(ind, reset) {
  currentMap = ind;
  while (allMaps[currentMap].cannotBeDisplayed == 1) currentMap = (currentMap + 1) % allMaps.length;
  imgCounter = (reset) ? allMaps[currentMap].maps.length - 1 : imgCounter - 1;
  isASwitch = false;
  nextMap();
  isASwitch = true;
  if (allMaps[currentMap].underlays.length > 0) {
    document.getElementById("topoimg").src = allMaps[currentMap].underlays[allMaps[currentMap].currentMap][0];
    document.getElementById("countiesimg").src = allMaps[currentMap].underlays[allMaps[currentMap].currentMap][1];
    document.getElementById("highwaysimg").src = allMaps[currentMap].underlays[allMaps[currentMap].currentMap][2];
    document.getElementById("citiesimg").src = allMaps[currentMap].underlays[allMaps[currentMap].currentMap][3];
  } else {
    document.getElementById("topoimg").src = "blank.png";
    document.getElementById("countiesimg").src = "blank.png";
    document.getElementById("highwaysimg").src = "blank.png";
    document.getElementById("citiesimg").src = "blank.png";
    document.getElementById("legendimg").src = "blank.png";
  }
  if (allMaps[currentMap].mapOptions.length > 1) {
    //mapSwitchButton.style.visibility = "visible";
    mapSwitchButton.children[1].innerHTML = allMaps[currentMap].mapOptions[(allMaps[currentMap].currentMap + 1) % allMaps[currentMap].mapOptions.length][0];
  } else {
    mapSwitchButton.style.visibility = "hidden";
  }
  if (allMaps[currentMap].index > -1) {
    //document.getElementById("removefromfavesButton").style.visibility = "visible";
    document.getElementById("addtofavesButton").style.visibility = "hidden";
  } else {
    document.getElementById("removefromfavesButton").style.visibility = "hidden";
    //document.getElementById("addtofavesButton").style.visibility = "visible";
  }
  if (actualShowAlerts && allMaps[currentMap].alert.innerHTML != null) {
    warningPane.style.background = allMaps[currentMap].alert.background;
    warningPane.innerHTML = allMaps[currentMap].alert.innerHTML;
    if (document.getElementById("back").style.display != "block") warningPane.style.visibility = "visible";
  } else
    warningPane.style.visibility = "hidden";
}


function switchMap() {
  currentMap = (currentMap + 1) % allMaps.length;
  while (allMaps[currentMap].cannotBeDisplayed == 1) currentMap = (currentMap + 1) % allMaps.length;
  if (allMaps.length > 1) imgCounter = allMaps[currentMap].maps.length - 1;
  if (allMaps[currentMap].underlays.length > 0) {
    document.getElementById("topoimg").src = allMaps[currentMap].underlays[allMaps[currentMap].currentMap][0];
    document.getElementById("countiesimg").src = allMaps[currentMap].underlays[allMaps[currentMap].currentMap][1];
    document.getElementById("highwaysimg").src = allMaps[currentMap].underlays[allMaps[currentMap].currentMap][2];
    document.getElementById("citiesimg").src = allMaps[currentMap].underlays[allMaps[currentMap].currentMap][3];
  } else {
    document.getElementById("topoimg").src = "blank.png";
    document.getElementById("countiesimg").src = "blank.png";
    document.getElementById("highwaysimg").src = "blank.png";
    document.getElementById("citiesimg").src = "blank.png";
    document.getElementById("legendimg").src = "blank.png";

  }
  isASwitch = false;
  if (allMaps.length > 1) nextMap();
  isASwitch = true;
  if (allMaps[currentMap].mapOptions.length > 1) {
    //mapSwitchButton.style.visibility = "visible";
    mapSwitchButton.children[1].innerHTML = allMaps[currentMap].mapOptions[(allMaps[currentMap].currentMap + 1) % allMaps[currentMap].mapOptions.length][0];
  } else {
    mapSwitchButton.style.visibility = "hidden";
  }
  if (allMaps[currentMap].index > -1) {
    //document.getElementById("removefromfavesButton").style.visibility = "visible";
    document.getElementById("addtofavesButton").style.visibility = "hidden";
  } else {
    document.getElementById("removefromfavesButton").style.visibility = "hidden";
    //document.getElementById("addtofavesButton").style.visibility = "visible";
  }
  if (actualShowAlerts && allMaps[currentMap].alert.innerHTML != null) {
    warningPane.style.background = allMaps[currentMap].alert.background;
    warningPane.innerHTML = allMaps[currentMap].alert.innerHTML;
    if (document.getElementById("back").style.display != "block") warningPane.style.visibility = "visible";
  } else
    warningPane.style.visibility = "hidden";
  updateMapNumber();
  updateHighlightBox();
  stop();
  if (active) start();
}


function switchMapUp() {
  currentMap = (currentMap + allMaps.length - 1) % allMaps.length;
  while (allMaps[currentMap].cannotBeDisplayed == 1) currentMap = (currentMap + allMaps.length - 1) % allMaps.length;

  if (allMaps.length > 1) imgCounter = (actualSwitchAtEnd) ? 0 : allMaps[currentMap].maps.length - 1;
  isASwitch = false;
  if (allMaps.length > 1) {
    if (actualSwitchAtEnd) lastMap();
    else nextMap();
  }
  isASwitch = true;
  if (allMaps[currentMap].mapOptions.length > 1) {
    //mapSwitchButton.style.visibility = "visible";
    mapSwitchButton.children[1].innerHTML = allMaps[currentMap].mapOptions[(allMaps[currentMap].currentMap + 1) % allMaps[currentMap].mapOptions.length][0];
  } else {
    mapSwitchButton.style.visibility = "hidden";
  }
  if (allMaps[currentMap].underlays.length > 0) {
    document.getElementById("topoimg").src = allMaps[currentMap].underlays[allMaps[currentMap].currentMap][0];
    document.getElementById("countiesimg").src = allMaps[currentMap].underlays[allMaps[currentMap].currentMap][1];
    document.getElementById("highwaysimg").src = allMaps[currentMap].underlays[allMaps[currentMap].currentMap][2];
    document.getElementById("citiesimg").src = allMaps[currentMap].underlays[allMaps[currentMap].currentMap][3];
  } else {
    document.getElementById("topoimg").src = "blank.png";
    document.getElementById("countiesimg").src = "blank.png";
    document.getElementById("highwaysimg").src = "blank.png";
    document.getElementById("citiesimg").src = "blank.png";
    document.getElementById("legendimg").src = "blank.png";

  }
  if (allMaps[currentMap].index > -1) {
    //document.getElementById("removefromfavesButton").style.visibility = "visible";
    document.getElementById("addtofavesButton").style.visibility = "hidden";
  } else {
    document.getElementById("removefromfavesButton").style.visibility = "hidden";
    //document.getElementById("addtofavesButton").style.visibility = "visible";
  }
  if (actualShowAlerts && allMaps[currentMap].alert.innerHTML != null) {
    warningPane.style.background = allMaps[currentMap].alert.background;
    warningPane.innerHTML = allMaps[currentMap].alert.innerHTML;
    warningPane.style.visibility = "visible";
  } else
    warningPane.style.visibility = "hidden";
  updateMapNumber();
  updateHighlightBox();
  stop();
  if (active) start();
}

// Udate after pressing the "Refresh" button
function update() {
  stop();
  warningPane.style.visibility = "visible";
  warningPane.style.background = "#283f8d";
  warningPane.innerHTML = "Loading Maps";
  for (i = 0; i < allMaps.length; i++) {
    allMaps[i].getMaps();
  }
  if (warningPane.innerHTML.indexOf("Loading") != -1) {
    lastUpdated = new Date();
    if (actualShowAlerts && allMaps[currentMap].alert.innerHTML != null) {
      warningPane.style.background = allMaps[currentMap].alert.background;
      warningPane.innerHTML = allMaps[currentMap].alert.innerHTML;
      if (document.getElementById("back").style.display != "block") warningPane.style.visibility = "visible";
    } else
      warningPane.style.visibility = "hidden";
  }
  if (active) start();
  if (actualStopped) reloadCurrentMap();

}

// Error printer, by Jason Trent and Jayson Vucovich
function drawUpdateFailure(message) {
  //fill in listings section
  document.getElementById('maps').innerHTML = "<h1>" + message + "</h1>";
}

// Toggle border on and off
function updateBorder() {

  var newBorder = actualBorder ? "visible" : "hidden";
  document.getElementById('topleft').style.visibility = newBorder;
  document.getElementById('middleleft').style.visibility = newBorder;
  document.getElementById('bottomleft').style.visibility = newBorder;
  document.getElementById('topright').style.visibility = newBorder;
  document.getElementById('topmiddle').style.visibility = newBorder;
  document.getElementById('bottommiddle').style.visibility = newBorder;
  document.getElementById('middleright').style.visibility = newBorder;
  document.getElementById('bottomright').style.visibility = newBorder;

}

function colorBorder(back) {
  document.getElementById('topleft').style.background = back ? "url(\"topleftb.png\") no-repeat top left" : "url(\"topleft.png\") no-repeat top left";
  document.getElementById('middleleft').style.background = back ? "url(\"leftmiddleb.png\") repeat top left" : "url(\"leftmiddle.png\") repeat top left";
  document.getElementById('bottomleft').style.background = back ? "url(\"bottomleftb.png\") no-repeat top left" : "url(\"bottomleft.png\") no-repeat top left";
  document.getElementById('topright').style.background = back ? "url(\"toprightb.png\") no-repeat top left" : "url(\"topright.png\") no-repeat top left";
  document.getElementById('topmiddle').style.background = back ? "url(\"topmiddleb.png\") repeat top left" : "url(\"topmiddle.png\") repeat top left";
  document.getElementById('bottommiddle').style.background = back ? "url(\"bottommiddleb.png\") repeat top left" : "url(\"bottommiddle.png\") repeat top left";
  document.getElementById('middleright').style.background = back ? "url(\"rightmiddleb.png\") repeat top left" : "url(\"rightmiddle.png\") repeat top left";
  document.getElementById('bottomright').style.background = back ? "url(\"bottomrightb.png\") no-repeat top left" : "url(\"bottomright.png\") no-repeat top left";

}

// Switch to the next radar
function gotoNextDetailMap() {
  var oldText = warningPane.innerText;
  allMaps[currentMap].gotoNextDetailMap();
  mapSwitchButton.children[1].innerHTML = allMaps[currentMap].mapOptions[(allMaps[currentMap].currentMap + 1) % allMaps[currentMap].mapOptions.length][0];
  if (actualStopped) reloadCurrentMap();
  warningPane.innerText = oldText;
}

// Functions from Apple Development Website for preference display
function enterflip(event) {
  document.getElementById('fliprollie').style.display = 'block';
}

function exitflip(event) {
  document.getElementById('fliprollie').style.display = 'none';
}

function entermaximize(event) {
  document.getElementById('maximizerollie').style.display = 'block';
}

function exitmaximize(event) {
  document.getElementById('maximizerollie').style.display = 'none';
}

var urlRequest = null;
// Search for city name entered
function getMapID(city) {
  if (actualSource == 0)
    return weatherChannel.getMapID(city);
  else
    return NOAA.getMapID(city);
}


// Open the weather page for the region, if in a specific ZIP/city
// or the weather channel page if on another map
function openWeather() {
  if (actualSource == 0) {
    if (allMaps[currentMap].map == 0) {
      widget.openURL("http://www.weather.com/weather/local/" + (validateZIPCode(allMaps[currentMap].regioncode) ? allMaps[currentMap].regioncode : allMaps[currentMap].locationID));
    } else
      widget.openURL("http://www.weather.com/");
  }
}

// Functions for resizing widget dynamically, from Apple's website

var growboxInset;
var oldX;
var oldY;

function mouseDown(event) {
  cannotHide = true;
  oldX = (windowWidth - 30);
  oldY = (windowHeight - 52);
  document.addEventListener("mousemove", mouseMove, true);
  document.addEventListener("mouseup", mouseUp, true);
  growboxInset = {
    x: (window.innerWidth - event.x),
    y: (window.innerHeight - event.y)
  };
  event.stopPropagation();
  event.preventDefault();
}

function mouseMove(event) {
  var x = event.x + growboxInset.x;
  var y = (actualSource == 0) ? (Math.floor((event.x + growboxInset.x - 30) * 0.675 + 52)) : (event.x + growboxInset.x);

  if (x < 390) {
    x = 390;
    y = (actualSource == 0) ? 295 : 390;
  }
  resizeWidgetTo(x, y);
  event.stopPropagation();
  event.preventDefault();
}

function mouseUp(event) {
  cannotHide = false;
  maximized = false;
  document.removeEventListener("mousemove", mouseMove, true);
  document.removeEventListener("mouseup", mouseUp, true);
  windowWidth = window.innerWidth;
  windowHeight = window.innerHeight;
  widget.setPreferenceForKey(windowWidth, createkey("windowwidth"));
  widget.setPreferenceForKey(windowHeight, createkey("windowheight"));
  event.stopPropagation();
  event.preventDefault();

  var xscale = (windowWidth - 30) / oldX;
  var yscale = (windowHeight - 52) / oldY;

  for (var i = 0; i < allMaps.length; i++)
    if (allMaps[i].highlight_box != null && allMaps[i].highlight_box[0] != null) {
      allMaps[i].highlight_box[0] = Math.floor(xscale * allMaps[i].highlight_box[0] - 12 * xscale + 12);
      allMaps[i].highlight_box[1] = Math.floor(yscale * allMaps[i].highlight_box[1] - 12 * yscale + 12);
    }

}

// Resize all necessary areas
function resizeWidgetTo(x, y) {
  var front = document.getElementById("front");
  var back = document.getElementById("back");
  var weatherURL = document.getElementById("weatherURL");
  var mapsArea = document.getElementById("maps");
  var topoArea = document.getElementById("topo");
  var topoImg = document.getElementById("topoimg");
  var countiesArea = document.getElementById("counties");
  var countiesImg = document.getElementById("countiesimg");
  var highwaysArea = document.getElementById("highways");
  var highwaysImg = document.getElementById("highwaysimg");
  var citiesArea = document.getElementById("cities");
  var citiesImg = document.getElementById("citiesimg");
  var legendArea = document.getElementById("legend");
  var legendImg = document.getElementById("legendimg");
  // "RESIZING TO " + x + "," + y );
  document.getElementById("resize").style.top = (y - 64) + "px";
  var newxv = x - 30;
  var newyv = y - 52;
  var newx = (newxv) + "px";
  var newy = (newyv) + "px";

  front.style.width = newx;
  front.style.height = newy;
  warningPane.style.width = newx;
  back.style.width = newx;
  back.style.height = (y - 42) + "px";
  mapsArea.style.width = newx;
  mapsArea.style.height = newy;
  topoArea.style.width = newx;
  topoArea.style.height = newy;
  topoImg.style.width = newx;
  topoImg.style.height = newy;
  countiesArea.style.width = newx;
  countiesArea.style.height = newy;
  countiesImg.style.width = newx;
  countiesImg.style.height = newy;
  highwaysArea.style.width = newx;
  highwaysArea.style.height = newy;
  highwaysImg.style.width = newx;
  highwaysImg.style.height = newy;
  citiesArea.style.width = newx;
  citiesArea.style.height = newy;
  citiesImg.style.width = newx;
  citiesImg.style.height = newy;
  legendArea.style.width = newx;
  legendArea.style.height = newy;
  legendImg.style.width = newx;
  legendImg.style.height = newy;
  weatherURL.style.top = (Math.floor((newyv) * 0.8353909) + 1) + "px";
  weatherURL.style.left = (Math.floor((newxv) * 0.875)) + "px";
  weatherURL.style.width = (Math.floor((newxv) * 0.07777778)) + "px";
  weatherURL.style.height = (Math.floor((newyv) * 0.102889)) + "px";
  if (document.getElementById("currentmap") != null) {
    document.getElementById("currentmap").style.width = newx;
    document.getElementById("currentmap").style.height = newy;
  } else
    document.getElementById("weatherURL").style.visibility = "hidden";


  if (allMaps[currentMap].highlight_box != null &&
    allMaps[currentMap].highlight_box[0] != null) {

    var xscale = newxv / (windowWidth - 30);
    var yscale = newyv / (windowHeight - 52);
    var highlight_box = document.getElementById("highlight_box");
    highlight_box.style.left = Math.floor(xscale * allMaps[currentMap].highlight_box[0] - 12 * xscale) + "px";
    highlight_box.style.top = Math.floor(yscale * allMaps[currentMap].highlight_box[1] - 12 * yscale) + "px";

  }

  document.getElementById("middleleft").style.height = (y - 60) + "px";
  document.getElementById("middleright").style.height = (y - 60) + "px";
  document.getElementById("bottommiddle").style.width = (x - 60) + "px";
  document.getElementById("topmiddle").style.width = (x - 60) + "px";
  document.getElementById("refresher").style.top = newy;
  statusPane.style.left = (Math.floor(0.25227272727272727272 * (newxv))) + "px";
  statusPane.style.height = (Math.ceil(0.03367003367003367003 * (newyv))) + "px";
  statusWidth = (Math.floor(0.50681818181818181818 * (newxv)));
  window.resizeTo(x, y);
}

// Click to zoom function
var maximized = false;

function maximize(event) {
  if (event.shiftKey)
    movebox(event);
  else {
    if (!maximized && windowWidth < 750)
      resizeWidgetTo(750, (actualSource == 0) ? 538 : 750);
    else
      resizeWidgetTo(windowWidth, windowHeight);
    maximized = !maximized;
  }
}

// Unhide and hide link over weather.com logo
function transparencyOff() {
  document.getElementById('weatherURL').style.opacity = "0.8";
}

function transparencyOn() {
  document.getElementById('weatherURL').style.opacity = "0.0";
}

function openHomepage() {
  widget.openURL("http://www.dividedsoul.net/RadarInMotion/");
}

function openiPhone() {
  widget.openURL("http://phobos.apple.com/WebObjects/MZStore.woa/wa/viewSoftware?id=292996643&mt=8");
}

function openPopularData() {
  widget.openURL("http://www.populardata.com/");
}

// Show the refresh button when the mouse is over it
function showRefresh() {
  if (warningPane.style.visibility == "visible")
    document.getElementById('fliprollieblack').style.display = 'block';
  if (document.getElementById("refresher").style.visibility) {
    document.getElementById("refresher").style.visibility = "visible";

    document.getElementById("refresh").style.visibility = "visible";
    if (actualStopped)
      document.getElementById("startButton").style.visibility = "visible";
    else
      document.getElementById("stopButton").style.visibility = "visible";
    document.getElementById("nextButton").style.visibility = "visible";
    document.getElementById("lastButton").style.visibility = "visible";
    if (allMaps[currentMap].index > -1)
      document.getElementById("removefromfavesButton").style.visibility = "visible";
    else
      document.getElementById("addtofavesButton").style.visibility = "visible";

    if (allMaps.length > 1)
      document.getElementById("mapnumber").style.visibility = "visible";


    if (allMaps[currentMap].mapOptions.length > 1) {
      mapSwitchButton.style.visibility = "visible";
    }
    document.getElementById("bottommiddle").style.bottom = "0px";
    document.getElementById("bottomright").style.bottom = "0px";
    document.getElementById("bottomleft").style.bottom = "0px";
    document.getElementById("middleleft").style.height = (window.innerHeight - 60) + "px";
    document.getElementById("middleright").style.height = (window.innerHeight - 60) + "px";
  }

}

// Hide the refresh button when the mouse is not over it
var cannotHide = false;

function hideRefresh() {
  if (!cannotHide) {
    mapSwitchButton.style.visibility = "hidden";
    document.getElementById("refresher").style.visibility = "hidden";
    document.getElementById("refresh").style.visibility = "hidden";
    document.getElementById("startButton").style.visibility = "hidden";
    document.getElementById("stopButton").style.visibility = "hidden";
    document.getElementById("nextButton").style.visibility = "hidden";
    document.getElementById("lastButton").style.visibility = "hidden";
    document.getElementById("map600Button").style.visibility = "hidden";
    document.getElementById("addtofavesButton").style.visibility = "hidden";
    document.getElementById("removefromfavesButton").style.visibility = "hidden";
    document.getElementById("mapnumber").style.visibility = "hidden";

    document.getElementById("bottommiddle").style.bottom = "22px";
    document.getElementById("bottomright").style.bottom = "22px";
    document.getElementById("bottomleft").style.bottom = "22px";
    document.getElementById("middleleft").style.height = (window.innerHeight - 82) + "px";
    document.getElementById("middleright").style.height = (window.innerHeight - 82) + "px";
  }
}

function validateZIPCode(code) {
  var index = 0;
  var isNumber = true;
  for (index = 0; index < code.length; index++) {
    if (code[index] < "0" || code[index] > "9" && code[index] < "A" || code[index] > "Z")
      isNumber = false;
  }
  return isNumber;

}

// Makes sure the animation speed or update time is a number
function validateNumber(speed) {
  var index = 0;
  var isNumber = true;

  for (index = 0; index < speed.length; index++)
    if (speed[index] < "0" || speed[index] > "9")
      isNumber = false;
  return isNumber;
}


// Create unique key for each instance of widget
function createkey(key) {
  return widget.identifier + "-" + key;
}

function addSpecificToFavorites(index) {
  allMaps[index].removeFromFavorites();
  allMaps[index].addToFavorites(actualFavorites++);
  if (window.widget)
    widget.setPreferenceForKey(actualFavorites, createkey("favorites"));
  document.getElementById("removefromfavesButton").style.visibility = "visible";
  document.getElementById("addtofavesButton").style.visibility = "hidden";
  updateMapNumber();
}

// Add to favorites
function addToFavorites() {
  allMaps[currentMap].removeFromFavorites();
  allMaps[currentMap].addToFavorites(actualFavorites++);
  if (window.widget)
    widget.setPreferenceForKey(actualFavorites, createkey("favorites"));
  document.getElementById("removefromfavesButton").style.visibility = "visible";
  document.getElementById("addtofavesButton").style.visibility = "hidden";
  updateMapNumber();
}

function removeFromFavorites() {
  stop();
  allMaps[currentMap].removeFromFavorites();
  actualFavorites--;
  if (window.widget)
    widget.setPreferenceForKey(actualFavorites, createkey("favorites"));

  if (allMaps.length > 1) {
    allMaps[currentMap] = null;
    allMaps.splice(currentMap, 1);
  }
  if (currentMap >= allMaps.length) currentMap = 0;
  if (allMaps[currentMap].mapOptions.length > 1) {
    mapSwitchButton.style.visibility = "visible";
    mapSwitchButton.children[1].innerHTML = allMaps[currentMap].mapOptions[(allMaps[currentMap].currentMap + 1) % allMaps[currentMap].mapOptions.length][0];
  } else {
    mapSwitchButton.style.visibility = "hidden";
  }
  if (allMaps[currentMap].index > -1) {
    document.getElementById("removefromfavesButton").style.visibility = "visible";
    document.getElementById("addtofavesButton").style.visibility = "hidden";
  } else {
    document.getElementById("removefromfavesButton").style.visibility = "hidden";
    document.getElementById("addtofavesButton").style.visibility = "visible";
  }
  if (allMaps[currentMap].underlays.length > 0) {
    document.getElementById("topoimg").src = allMaps[currentMap].underlays[allMaps[currentMap].currentMap][0];
    document.getElementById("countiesimg").src = allMaps[currentMap].underlays[allMaps[currentMap].currentMap][1];
    document.getElementById("highwaysimg").src = allMaps[currentMap].underlays[allMaps[currentMap].currentMap][2];
    document.getElementById("citiesimg").src = allMaps[currentMap].underlays[allMaps[currentMap].currentMap][3];
  } else {
    document.getElementById("topoimg").src = "blank.png";
    document.getElementById("countiesimg").src = "blank.png";
    document.getElementById("highwaysimg").src = "blank.png";
    document.getElementById("citiesimg").src = "blank.png";
    document.getElementById("legendimg").src = "blank.png";

  }
  updateMapNumber();
  if (active) start();
  if (actualStopped) reloadCurrentMap();
}



// Display back of widget with preferences
function showPrefs() {
  var updatefreq = document.getElementById("updatefreq");

  stop();

  // Put elements of form into variables
  document.getElementById("source").selectedIndex = actualSource;


  // Make sure white circle disappears
  exitflip(event);

  if (window.widget)
    widget.prepareForTransition("ToBack");
  warningPane.style.visibility = "hidden";
  document.getElementById("front").style.display = "none";
  document.getElementById("back").style.display = "block";
  document.getElementById("settings").style.display = "block";
  colorBorder(true);
  nextPrefs(0);

  if (window.widget)
    setTimeout('widget.performTransition();', 0);

  // If widget is not large enough to display preferences, resize it
  if (windowWidth < 390 && !maximized)
    resizeWidgetTo(390, (actualSource == 0) ? 295 : 390);

  // Load values into form
  document.getElementById("speed").value = actualAnimationSpeed;
  if (actualUpdateFrequency != 0) {
    document.getElementById("autoupdate").checked = true;
    updatefreq.value = actualUpdateFrequency;
  } else {
    document.getElementById("autoupdate").checked = false;
    updatefreq.value = "";
  }

  if (actualActivateUpdate > -1) {
    document.getElementById("activateupdate").checked = true;
    document.getElementById("activateupdatefreq").value = actualActivateUpdate;
  }
  autoIsChecked();
  activateIsChecked();

  if (actualSaveMaps > 0) {
    document.getElementById("savemaps").checked = true;
    document.getElementById("savetime").value = actualSaveMaps / 3600000;
  }
  saveIsChecked();

  if (actualRotateTime > 0) {
    document.getElementById("rotatemaps").checked = true;
    document.getElementById("rotatetime").value = actualRotateTime;
  }
  rotateIsChecked();

  if (actualHold) document.getElementById("hold").checked = true;
  if (actualForecastHalf) document.getElementById("forecastspeed").checked = true;
  if (actualBorder) document.getElementById("border").checked = true;

  if (actualShowAlerts) document.getElementById("showalerts").checked = true;
  checkForNewVersion();
  if (allMaps[allMaps.length - 1].index == -1) {
    var lastMap = allMaps[allMaps.length - 1];
    document.getElementById("zipcode").value = lastMap.regioncode;
    document.getElementById("region").selectedIndex = lastMap.region;
    selectedRegion();
    document.getElementById("maptype").value = lastMap.map;
    selectedIsZip();
  } else {
    document.getElementById("zipcode").value = "";
    document.getElementById("region").selectedIndex = 0;
    selectedRegion();
    selectedIsZip();
  }
}

// Set all changes to preferences and flip the widget back over
function hidePrefs() {
  var front = document.getElementById("front");
  var back = document.getElementById("back");
  var settings = document.getElementById("settings");
  var speed = document.getElementById("speed");
  var zipcode = document.getElementById("zipcode");
  var autoupdate = document.getElementById("autoupdate");
  var updatefreq = document.getElementById("updatefreq");
  var maptype = document.getElementById("maptype");
  var region = document.getElementById("region");
  var hold = document.getElementById("hold");
  var forecastspeed = document.getElementById("forecastspeed");
  var border = document.getElementById("border");
  var activateupdate = document.getElementById("activateupdate");
  var activateupdatefreq = document.getElementById("activateupdatefreq");
  var savemaps = document.getElementById("savemaps");
  var savetime = document.getElementById("savetime");
  var rotatemaps = document.getElementById("rotatemaps");
  var rotatetime = document.getElementById("rotatetime");

  var newZipCode = zipcode.value;

  var updateNeeded = false;

  // New Animation Speed
  if (speed.value != actualAnimationSpeed && validateNumber(speed.value)) {
    setAnimationSpeed(speed.value);
  }

  // New Animation Speed
  if (rotatetime.value != actualRotateTime && validateNumber(rotatetime.value)) {
    setRotationTime(rotatetime.value);
  }

  // New Update Frequency
  if (updatefreq.value != actualUpdateFrequency && autoupdate.checked && validateNumber(updatefreq.value)) {
    setUpdateFrequency(updatefreq.value);
  } else if (updatefreq.value != actualUpdateFrequency && !autoupdate.checked) {
    setUpdateFrequency(0);
  }

  // New value for holding last map
  if (hold.checked != actualHold) {
    actualHold = hold.checked;
    if (window.widget)
      widget.setPreferenceForKey(actualHold, createkey("hold"));
  }

  // New value for forecast speed
  if (forecastspeed.checked != actualForecastHalf) {
    actualForecastHalf = forecastspeed.checked;
    if (window.widget)
      widget.setPreferenceForKey(actualForecastHalf, createkey("forecastspeed"));
  }

  // New value for showing alerts
  if (showalerts.checked != actualShowAlerts) {
    actualShowAlerts = showalerts.checked;
    if (window.widget)
      widget.setPreferenceForKey(actualShowAlerts, createkey("showalerts"));
    updateNeeded = true;
    /*    if(actualShowAlerts && !(((newZipCode != actualZipCode && newZipCode != "Invalid location!") || (maptype.value != actualMap && maptype.value==0))))
            updateNeeded = true;
          else if(warningPane.innerHTML.indexOf("Map") == -1)
          {
                warningPane.style.visibility = "hidden";
                warningPane.innerHTML = "";
          }*/
  }

  // New value for border
  if (border.checked != actualBorder) {
    actualBorder = border.checked;

    if (window.widget)
      widget.setPreferenceForKey(actualBorder, createkey("hasborder"));
    updateBorder();
  }

  // Change in update on activation
  if (activateupdate.checked && activateupdatefreq.value != actualActivateUpdate) {
    actualActivateUpdate = activateupdatefreq.value;
    if (window.widget)
      widget.setPreferenceForKey(actualActivateUpdate, createkey("activateupdate"));
  } else if (!activateupdate.checked && actualActivateUpdate != -1) {
    actualActivateUpdate = -1;
    if (window.widget)
      widget.setPreferenceForKey(actualActivateUpdate, createkey("activateupdate"));
  }

  // Change in saving maps
  if (savemaps.checked && validateNumber(savetime.value) && savetime.value * 3600000 != actualSaveMaps) {
    var oldSaveMaps = actualSaveMaps;
    actualSaveMaps = savetime.value * 3600000;
    if (window.widget)
      widget.setPreferenceForKey(actualSaveMaps, createkey("savemaps"));

    // Clear maps if switch from not saving to saving
    if (oldSaveMaps == 0) {
      clearMaps();
      createDirectory();
      if (widget.system("/bin/ls \"" + folder + "\"", null).errorString) {

        warningPane.style.visibility = "visible";
        warningPane.style.background = "#283f8d";
        warningPane.innerHTML = "Warning: Cannot create maps directory";
        noProblem = false;
        actualSaveMaps = 0;
        if (window.widget)
          widget.setPreferenceForKey(actualSaveMaps, createkey("savemaps"));
        setTimeout("document.getElementById('warning').style.visibility = 'hidden'", 3000);
      }
      updateNeeded = true;
    } else
      for (k = 0; k < allMaps.length; k++) allMaps[k].removeOldMaps();

  } else if (!savemaps.checked && actualSaveMaps > 0) {
    clearMaps();
    actualSaveMaps = 0;
    if (window.widget)
      widget.setPreferenceForKey(actualSaveMaps, createkey("savemaps"));
    updateNeeded = true;
  }

  var newMap = allMaps.length == 0 || allMaps[allMaps.length - 1].region != region.selectedIndex ||
    allMaps[allMaps.length - 1].map != maptype.value ||
    (allMaps[allMaps.length - 1].regioncode != newZipCode && newZipCode != "");

  // Load the map selected, if it is valid
  if (maptype.value != 0 && newMap) {
    for (i = 0; i < allMaps.length; i++)
      if (allMaps[i].index == -1) allMaps.splice(i, 1);
    allMaps[allMaps.length] = new Maps(region.selectedIndex, maptype.value, "");
    warningPane.style.visibility = "visible";
    warningPane.style.background = "#283f8d";
    warningPane.innerHTML = "Loading Maps";
    allMaps[allMaps.length - 1].getMaps();
    allMaps[allMaps.length - 1].addToFavorites(-1);
    warningPane.style.visibility = "hidden";
    //showRefresh();
  }

  var noProblem = true;

  // New city
  if (newZipCode != "Invalid location!" && maptype.value == 0 && newZipCode != "" && newMap) {
    // Value is a zip code
    if (validateZIPCode(newZipCode)) {
      for (i = 0; i < allMaps.length; i++)
        if (allMaps[i].index == -1) allMaps.splice(i, 1);
      allMaps[allMaps.length] = new Maps(region.selectedIndex, maptype.value, newZipCode);
      if (allMaps[allMaps.length - 1].mapOptions[0]) {
        warningPane.style.visibility = "visible";
        warningPane.style.background = "#283f8d";
        warningPane.innerHTML = "Loading Maps";
        allMaps[allMaps.length - 1].getMaps();
        allMaps[allMaps.length - 1].addToFavorites(-1);
        warningPane.style.visibility = "hidden";
      } else {
        allMaps.splice(allMaps.length - 1, 1);
        zipcode.value = "Invalid location!";
        noProblem = false;
      }
      //showRefresh();
    }

    // City name entered; need to hunt for the city in database
    else {
      oldZIP = newZipCode;
      newZipCode = getMapID(newZipCode);
      if (newZipCode != null) {
        for (i = 0; i < allMaps.length; i++)
          if (allMaps[i].index == -1) allMaps.splice(i, 1);
        allMaps[allMaps.length] = new Maps(region.selectedIndex, maptype.value, newZipCode);
        warningPane.style.visibility = "visible";
        warningPane.style.background = "#283f8d";
        warningPane.innerHTML = "Loading Maps";
        allMaps[allMaps.length - 1].getMaps();
        allMaps[allMaps.length - 1].addToFavorites(-1);
        warningPane.style.visibility = "hidden";
        //showRefresh();
      } else {
        zipcode.value = "Invalid location!";
        noProblem = false;
      }
    }
  }


  // If a city name was entered and city was found or now changes to city were made,
  // we can update and flip over
  if (noProblem) {
    if (window.widget)
      widget.prepareForTransition("ToFront");

    back.style.display = "none";
    settings.style.display = "none";
    if (allMaps[currentMap].mapOptions.length > 1) {
      mapSwitchButton.style.visibility = "visible";
      mapSwitchButton.children[1].innerHTML = allMaps[currentMap].mapOptions[(allMaps[currentMap].currentMap + 1) % allMaps[currentMap].mapOptions.length][0];
    } else {
      mapSwitchButton.style.visibility = "hidden";
    }
    if (warningPane.innerHTML != "" && warningPane.innerHTML.indexOf("Loading") == -1)
      warningPane.style.visibility = "visible";
    front.style.display = "block";
    colorBorder(false);
    document.getElementById("backAddButtonText").innerHTML = "";
    if (window.widget)
      setTimeout('widget.performTransition();', 0);
    if (!maximized) resizeWidgetTo(windowWidth, windowHeight);
    if (actualSource != document.getElementById("source").value) {
      switchSource();
      resizeWidgetTo(windowWidth, windowHeight);
    } else if (updateNeeded) update();
    switchToMap(currentMap, false);
    updateMapNumber();
    if (active) start();
  }

}

function addToFavoritesBack() {
  var zipcode = document.getElementById("zipcode");
  var maptype = document.getElementById("maptype");
  var region = document.getElementById("region");
  var newZipCode = zipcode.value;
  var newMap = allMaps[allMaps.length - 1].region != region.selectedIndex ||
    allMaps[allMaps.length - 1].map != maptype.value ||
    (allMaps[allMaps.length - 1].regioncode != newZipCode && newZipCode != "");

  if (!newMap && allMaps[allMaps.length - 1].index == -1)
    addSpecificToFavorites(allMaps.length - 1);

  // Load the map selected, if it is valid
  if (maptype.value != 0 && newMap) {
    for (i = 0; i < allMaps.length; i++)
      if (allMaps[i].index == -1) allMaps.splice(i, 1);
    allMaps[allMaps.length] = new Maps(region.selectedIndex, maptype.value, "");
    warningPane.style.visibility = "visible";
    warningPane.style.background = "#283f8d";
    warningPane.innerHTML = "Loading Maps";
    allMaps[allMaps.length - 1].getMaps();
    addSpecificToFavorites(allMaps.length - 1);
    warningPane.style.visibility = "hidden";
    //showRefresh();
  }

  var noProblem = true;

  document.getElementById("backAddButtonText").innerHTML = "Adding Map.";
  // New city
  if (newZipCode != "Invalid location!" && maptype.value == 0 && newZipCode != "" && newMap) {
    // Value is a zip code
    if (validateZIPCode(newZipCode) || actualSource == 1) {
      for (i = 0; i < allMaps.length; i++)
        if (allMaps[i].index == -1) allMaps.splice(i, 1);
      allMaps[allMaps.length] = new Maps(region.selectedIndex, maptype.value, newZipCode);
      if (allMaps[allMaps.length - 1].mapOptions[0]) {
        warningPane.style.visibility = "visible";
        warningPane.style.background = "#283f8d";
        warningPane.innerHTML = "Loading Maps";
        allMaps[allMaps.length - 1].getMaps();
        addSpecificToFavorites(allMaps.length - 1);
        warningPane.style.visibility = "hidden";
      } else {
        allMaps.splice(allMaps.length - 1, 1);
        zipcode.value = "Invalid location!";
        noProblem = false;
      }
      //showRefresh();
    }

    // City name entered; need to hunt for the city on weather.com
    else {
      oldZIP = newZipCode;
      newZipCode = getMapID(newZipCode);
      if (newZipCode != undefined) {
        for (i = 0; i < allMaps.length; i++)
          if (allMaps[i].index == -1) allMaps.splice(i, 1);
        allMaps[allMaps.length] = new Maps(region.selectedIndex, maptype.value, newZipCode);
        warningPane.style.visibility = "visible";
        warningPane.style.background = "#283f8d";
        warningPane.innerHTML = "Loading Maps";
        allMaps[allMaps.length - 1].getMaps();
        addSpecificToFavorites(allMaps.length - 1);
        warningPane.style.visibility = "hidden";
        //showRefresh();
      } else {
        zipcode.value = "Invalid location!";
        noProblem = false;
      }
    }
  }

  if (noProblem) {
    zipcode.value = "";
    region.selectedIndex = 0;
    selectedRegion();
    maptype.selectedIndex = 0;
    document.getElementById("backAddButtonText").innerHTML = "Map Added!";
    setTimeout("document.getElementById('backAddButtonText').innerHTML = ''", 3000);
  }

}




function selectedIsZip() {
  if (document.getElementById("maptype").value == 0)
    document.getElementById("zipcode").disabled = false;
  else
    document.getElementById("zipcode").disabled = true;
}

// Make corresponding time box active if update automatically is checked
function autoIsChecked() {
  if (document.getElementById("autoupdate").checked) {
    document.getElementById("updatefreq").disabled = false;
    if (actualUpdateFrequency > 0) document.getElementById("updatefreq").value = actualUpdateFrequency;
    else document.getElementById("updatefreq").value = defaultUpdateFrequency;
  } else {
    document.getElementById("updatefreq").disabled = true;
    document.getElementById("updatefreq").value = "";
  }
}

// Make corresponding time box active if save maps is checked
function saveIsChecked() {
  if (document.getElementById("savemaps").checked) {
    document.getElementById("savetime").disabled = false;
    if (actualSaveMaps > 0)
      document.getElementById("savetime").value = actualSaveMaps / 3600000;
    else
      document.getElementById("savetime").value = 5;
  } else {
    document.getElementById("savetime").disabled = true;
    document.getElementById("savetime").value = "";
  }
}

// Make corresponding time box active if update on activation is checked
function activateIsChecked() {
  if (document.getElementById("activateupdate").checked) {
    document.getElementById("activateupdatefreq").disabled = false;
    if (actualActivateUpdate > -1)
      document.getElementById("activateupdatefreq").value = actualActivateUpdate;
    else document.getElementById("activateupdatefreq").value = 10;
  } else {
    document.getElementById("activateupdatefreq").disabled = true;
    document.getElementById("activateupdatefreq").value = "";
  }
}

// Make corresponding time box active if save maps is checked
function rotateIsChecked() {
  if (document.getElementById("rotatemaps").checked) {
    document.getElementById("rotatetime").disabled = false;
    if (actualRotateTime > 0)
      document.getElementById("rotatetime").value = actualRotateTime;
    else
      document.getElementById("rotatetime").value = defaultRotateTime;
  } else {
    document.getElementById("rotatetime").disabled = true;
    document.getElementById("rotatetime").value = "";
  }
}

function rotateOnEndIsChecked() {
  actualSwitchAtEnd = document.getElementById("rotatemapsonend").checked;
  widget.setPreferenceForKey(actualSwitchAtEnd, createkey("switchatend"));
}


// Change state of border
function borderIsChecked() {
  oldBorder = actualBorder;
  actualBorder = document.getElementById("border").checked;
  updateBorder();
  actualBorder = oldBorder;
}

var versionRequest;
// Download version.txt from website and check version numbers
function checkForNewVersion() {
  var versionURL = "http://www.dividedsoul.net/RadarInMotion/version.txt";
  if (versionURL) {
    if (versionRequest)
      versionRequest.abort();
    versionRequest = new XMLHttpRequest();
    versionRequest.onreadystatechange = getVersionNumber;
    versionRequest.open("GET", versionURL, false);
    versionRequest.send(null);
  }
}

// If new version is available, display link and notice
function getVersionNumber() {
  try {
    if (versionRequest.readyState == 4 && versionRequest.status == 200) {
      if (versionRequest.responseText != versionNum) {
        document.getElementById("note").style.color = "#ff6666";
        document.getElementById("note").innerHTML = "New Version Available!<br /><span class=\"newversion\" onclick=\"widget.openURL('http://www.dividedsoul.net/RadarInMotion/');\">See New Features and Download</span>";
      }

    } else if (versionRequest.readyState == 4) {
      versionRequest.abort();
      versionRequest = null;
    }
  } catch (exception) {
    drawUpdateFailure("Failed to load URL: " + exception);
  }
}

// Display advanced preferences
function toggleAdvanced() {
  document.getElementById("advanced").style.display = "block";
  document.getElementById("advancedtext").style.display = "none";
}

function setRotationTime(speed) {
  actualRotateTime = speed;
  if (window.widget)
    widget.setPreferenceForKey(actualRotateTime, createkey("rotatetime"));
  clearInterval(switchTimer);
  switchTimer = null;
  if (actualRotateTime > 0 && switchTimer == null)
    switchTimer = setInterval("switchMap();", 1000 * actualRotateTime);
}

// Change how fast the images are reset
function setAnimationSpeed(speed) {
  actualAnimationSpeed = speed;
  if (window.widget)
    widget.setPreferenceForKey(actualAnimationSpeed, createkey("animationspeed"));
}

// Change how often the maps reload
function setUpdateFrequency(freq) {
  actualUpdateFrequency = freq;
  if (window.widget)
    widget.setPreferenceForKey(actualUpdateFrequency, createkey("updatefrequency"));
  if (updateTimer != null) {
    clearInterval(updateTimer);
    updateTimer = null;
  }
  if (updateTimer == null && actualUpdateFrequency != 0)
    updateTimer = setInterval("update();", actualUpdateFrequency * 60 * 1000);
}

function keyPress(e) {
  if (e.which == 32 &&
    (!document.getElementById("back").style.display || document.getElementById("back").style.display == "none")) {
    if (actualStopped) manualStart();
    else manualStop();
  } else if (e.which == 13 && document.getElementById("back").style.display == "block")
    hidePrefs();

  else {
    if (e.which == 38) switchMapUp();
    else if (e.which == 40) switchMap();
    else if (e.which == 37) lastMap();
    else if (e.which == 39) nextMap();
  }
}

function switchSource() {
  switchSourceTo((actualSource + 1) % 2);
}

function switchSourceTo(source) {
  warningPane.style.visibility = "visible";
  warningPane.style.background = "#283f8d";
  warningPane.innerHTML = "Switching Sources";
  actualSource = source;
  if (window.widget)
    widget.setPreferenceForKey(actualSource, createkey("source"));
  if (actualSource == 0) {
    windowWidth++;
    windowHeight = windowWidth * 0.675 + 52;
    document.getElementById("region").innerHTML = "<option>United States</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Northeast</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--East Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Southeast</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--North Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--South Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Northwest</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--West Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Southwest</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Other</option>" +
      "<option>Africa &amp; Middle East</option>" +
      "<option>Asia</option>" +
      "<option>Australia</option>" +
      "<option>Central America</option>" +
      "<option>Europe</option>" +
      "<option>Global</option>" +
      "<option>North America</option>" +
      "<option>Pacific</option>" +
      "<option>South America</option>";
    document.getElementById("weatherURL").style.visibility = "visible";

  } else {
    windowWidth++;
    windowHeight = windowWidth;
    document.getElementById("region").innerHTML = "<option>United States</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Northeast</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--East Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Southeast</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--North Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--South Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Northwest</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--West Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Southwest</option>";
    document.getElementById("weatherURL").style.visibility = "hidden";
  }
  warningPane.style.visibility = "visible";
  //resizeWidgetTo(windowWidth,windowHeight);
  selectedRegion();
  warningPane.style.visibility = "visible";
  clearMaps();
  warningPane.style.visibility = "visible";

  for (var e = 0; e < allMaps.length; e++) {
    var oldWasFave = allMaps[e].index;
    allMaps[e] = new Maps(allMaps[e].region, allMaps[e].map, allMaps[e].regioncode);
    warningPane.style.visibility = "visible";
    warningPane.innerHTML = "Switching Sources";
    for (j = 0; j < e; j++)
      warningPane.innerHTML += ".";
    if (oldWasFave > -1) allMaps[e].addToFavorites(oldWasFave);
  }
  update();
  var availableMaps = 0;
  for (i = 0; i < allMaps.length; i++)
    if (allMaps[i].cannotBeDisplayed == 0) availableMaps++;
  if (availableMaps > 0) {
    switchToMap(0, true);
    widget.setPreferenceForKey(windowWidth, createkey("windowwidth"));
    widget.setPreferenceForKey(windowHeight, createkey("windowheight"));
    warningPane.style.visibility = "hidden";
  } else {
    warningPane.style.visibility = "visible";
    warningPane.style.background = "#283f8d";
    warningPane.innerHTML = "Warning: No maps to display with that source";
    switchSourceTo(0);
  }

}

function setRegionSelect() {
  if (document.getElementById("source").selectedIndex == 0) {
    document.getElementById("region").innerHTML = "<option>United States</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Northeast</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--East Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Southeast</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--North Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--South Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Northwest</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--West Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Southwest</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Other</option>" +
      "<option>Africa &amp; Middle East</option>" +
      "<option>Asia</option>" +
      "<option>Australia</option>" +
      "<option>Central America</option>" +
      "<option>Europe</option>" +
      "<option>Global</option>" +
      "<option>North America</option>" +
      "<option>Pacific</option>" +
      "<option>South America</option>";
  } else {
    document.getElementById("region").innerHTML = "<option>United States</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Northeast</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--East Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Southeast</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--North Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--South Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Northwest</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--West Central</option>" +
      "<option>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;--Southwest</option>";
  }

  //document.getElementById("region").disabled = document.getElementById("source").selectedIndex == 1;
  document.getElementById("region").selectedIndex = 0;
  selectedRegion();
}

function nextPrefs(setting) {
  for (i = 0; i <= 3; i++) {
    document.getElementById("settings" + i).style.display = "none";
    document.getElementById("nav-" + i).children[0].style.background = "#000000";
  }
  document.getElementById("settings" + setting).style.display = "block";
  document.getElementById("nav-" + setting).children[0].style.background = "#333333";
}

function updateMapNumber() {
  var mapNumber = document.getElementById("mapnumber");
  //document.getElementById("mapnumber").innerHTML = ""
  if (allMaps.length > 1) {
    mapNumber.children[1].innerHTML = (currentMap + 1) + "/" + allMaps.length;
    //document.getElementById("nextRegionButton").style.visibility = "visible";
    //document.getElementById("lastRegionButton").style.visibility = "visible";
    mapNumber.style.visibility = "visible";
  } else {
    //document.getElementById("nextRegionButton").style.visibility = "hidden";
    //document.getElementById("lastRegionButton").style.visibility = "hidden";
    mapNumber.style.visibility = "hidden";
  }
}

function scrolling(event) {
  if (event.wheelDelta < -200) switchMap();
  else if (event.wheelDelta > 200) switchMapUp();
}

function movebox(event) {
  if (event.shiftKey) {
    moveboxTo(event.x, event.y);
    allMaps[currentMap].highlight_box = [event.x, event.y];
    allMaps[currentMap].saveHighlight();
  }
}

function moveboxTo(x, y) {
  var highlight_box = document.getElementById("highlight_box");
  highlight_box.style.left = (x - 12) + "px";
  highlight_box.style.top = (y - 12) + "px";
  highlight_box.style.visibility = "visible";
}

function hidebox(event) {
  document.getElementById("highlight_box").style.visibility = "hidden";
  allMaps[currentMap].highlight_box = null;
  allMaps[currentMap].saveHighlight();
}

function updateHighlightBox() {
  if (allMaps[currentMap].highlight_box != null && allMaps[currentMap].highlight_box[0] != null && allMaps[currentMap].highlight_box[0] != undefined) {
    moveboxTo(allMaps[currentMap].highlight_box[0], allMaps[currentMap].highlight_box[1]);
  } else
    hidebox(null);
}
